@@ -917,6 +917,9 @@ vsf_sysutil_isdigit(int the_char)
 char*
 vsf_sysutil_getcwd(char* p_dest, const unsigned int buf_size)
 {
+  if (buf_size == 0) {
+    return p_dest;
+  }
   char* p_retval = getcwd(p_dest, buf_size);
   p_dest[buf_size - 1] = '\0';
   return p_retval;
@@ -1512,6 +1515,9 @@ vsf_sysutil_unlock_file(int fd)
 int
 vsf_sysutil_readlink(const char* p_filename, char* p_dest, unsigned int bufsiz)
 {
+  if (bufsiz == 0) {
+    return -1;
+  }
   int retval = readlink(p_filename, p_dest, bufsiz - 1);
   if (retval < 0)
   {
